/*
 * hfwload - firmware loader for hermes wavelan cards
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "hermesfw.h"
#include "pdar.h"
#include "hfw.h"
#include "hermes_mem.h"

unsigned long pagesize=HERMES_AUX_PAGESIZE;
unsigned long pages=HERMES_AUX_PAGES;

int main(int argc, char **argv) {
	unsigned char pda[PDA_SIZE_MAX], *macaddr, buf[40], *opda=NULL;
	FILE *opdaf=NULL;
	struct hfw *hfw;
	int fd, verbose=0, res;
	
	for(res=3;res<argc;res++) {
		if(!strcmp(argv[res],"-v"))
			verbose++;
		if(!strcmp(argv[res],"-o")) {
			if(res+1<argc) {
				res++;
				if(!strcmp(argv[res],"-"))
					opdaf=stdin;
				else {
					opdaf=fopen(argv[res],"r");
					if(!opdaf) {
						fprintf(stderr,"%s: can't read PDA from file %s\n",argv[0],argv[res]);
						return 0;
					}
				} /* opda not from stdin */
			} /* next arg valid */
			else {
				fprintf(stderr,"%s: no filename for PDA given!\n",argv[0]);
				return 0;
			}
		} /* opda */
	}
	
	if(argc<3) {
		fprintf(stderr,"usage: %s ethX <firmware.hfw> (-o <opda>) (-v (-v (-v)))\n",argv[0]);
		fprintf(stderr,"see docs and http://www.comteam.at/~alex/phpBB2/index.php for more infos\n");
		return 0;
	}
	
	strcpy(pda,"/dev/orinoco/");
	strcat(pda,argv[1]);
	strcat(pda,"_mem");

	if((fd=open(pda,O_RDWR))<0) {
		fprintf(stderr,"can't open %s\n",pda);
		return -1;
	}
	
	if(!(hfw=read_hfw(argv[2]))) {
		fprintf(stderr,"error reading/parsing %s\n",argv[2]);
		return -1;
	}
	
	if(verbose)
		dump_segments(hfw->segms);
	if(verbose>2)
		write_hfw_plugs(stderr,hfw->plugs);
	
	if(read_pda_from(fd,PAGE_TO_OFFSET(0x7200),pda)) {
		fprintf(stderr,"error reading PDA (PAGE 0x%04X POF 0 == OFFSET 0x%08lX) from %s\n",
						0x7200,PAGE_TO_OFFSET(0x7200),argv[1]);
		return -1;
	}
	
	if(verbose>1)
		dump_pda(pda,PDA_SIZE_MAX);
	
	if(!opdaf) {
		macaddr=lookup_pda(PDA_RECORD_MACADDR, 6, pda);
		if(!macaddr) {
			fprintf(stderr,"WAAH! no MACAddr. in your PDA found!\n");
			return -1;
		}
		sprintf(buf, "/etc/hermesap/%02X:%02X:%02X:%02X:%02X:%02X.opda",
					macaddr[0], macaddr[1], macaddr[2], macaddr[3], macaddr[4], macaddr[5]);
		opdaf=fopen(buf, "r");
	} /* use default opda location */
	
	if(opdaf) {
		opda=read_txt_pda(opdaf);
		if(!opda) {
			fprintf(stderr,"overriding PDA invalid!\n");
			return 0;
		}
	}
	
	if(opda) {
		printf("overriding PDA entries\n");
		if (verbose)
			dump_pda(opda, PDA_SIZE_MAX);
	}
	
	if(hfw_plug_records(hfw,opda,pda,verbose)) {
		fprintf(stderr,"plugging failed - aborting\n");
		fprintf(stderr,"try increasing the verbosity for more details!\n");
		fprintf(stderr,"you mind find help here:\nhttp://www.comteam.at/~alex/phpBB2/index.php\n");
		return -1;
	}
	if(!hfw->entry)
		return 0;
#ifndef TEST
	if(ioctl(fd,ORINOCO_PREPARE_RAMDL,hfw->entry)) {
		fprintf(stderr,"ORINOCO_PREPARE_RAMDL ioctl failed - aborting\n");
		fprintf(stderr,"maybe you forgot ifconfig %s down ?\n",argv[1]);
		return -1;
	}
#endif
	
	res=hfw_upload(hfw,fd);
	if(res) {
		fprintf(stderr,"hfw_upload failed: %d!\n",res);
		fprintf(stderr,"try increasing the verbosity for more details!\n");
		fprintf(stderr,"you mind find help here:\nhttp://www.comteam.at/~alex/phpBB2/index.php\n");
	}
	else {
		printf("booting firmware...");
		fflush(stdout);
	}
	
#ifndef TEST
	if(ioctl(fd,ORINOCO_FINISH_RAMDL,0)) {
		printf("failed!\n");
		fprintf(stderr,"ORINOCO_FINISH_RAMDL ioctl failed\n");
		fprintf(stderr,"maybe already a RAM firmware running?\n");
		return -1;
	}
#endif

	printf("OK\n");
	
	return 0;
}
