/*
 * pda related functions for hfwload
 * (firmware loader for hermes wavelan cards)
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "pdar.h"
#include "hfw.h"

int verify_pda(unsigned char *data, int size) {
	unsigned char *pd=data;
	u16 len, code;
	int res=-1;
	
	while((pd-data)<size) {
		memcpy(&len,pd,sizeof(u16));
		len=((le16_to_host(len))<<1)-2;
		if(len>(size-4-(pd-data)))
			return -1;
		memcpy(&code,pd+sizeof(u16),sizeof(u16));
		code=le16_to_host(code);
		if(!code) {
			if(len==2)
				return 0;
			else
				return -1;
		}
		pd+=len+4;
	}
	return res;
}

int read_pda_from(int fd, off_t offset, unsigned char *data) { 
	int res;
	
	if(lseek(fd,offset,SEEK_SET)!=offset)
		return -1;
	else if((res=read(fd,data,PDA_SIZE_MAX))<6)
		return 1;
	else
		return verify_pda(data,res);
}

void dump_pda(unsigned char *data, int size) {
	unsigned char *pd=data;
	u16 len, code;
	int count;
	
	while((pd-data)<size) {
		memcpy(&len,pd,sizeof(u16));
		len=((le16_to_host(len))<<1)-2;
		if(len>(size-4-(pd-data)))
			return;
		memcpy(&code,pd+sizeof(u16),sizeof(u16));
		code=le16_to_host(code);
		if(!code) 
			return;
		printf("%04X: ",code);
		pd+=4;
		for(count=0;count<len;count+=2)
			printf("%02X%02X ",pd[count],pd[count+1]);
		printf("\n");
		pd+=len;
	}
}

unsigned char * lookup_pda (unsigned short code, unsigned short len, unsigned char *pd) {
	u16 len2, code2;
	
	if((!pd) || (!code) || (!len) || (len&1))
		return NULL;
	
	while(1) {
		memcpy(&len2,pd,sizeof(u16));
		len2=((le16_to_host(len2))<<1)-2;
		memcpy(&code2,pd+sizeof(u16),sizeof(u16));
		code2=le16_to_host(code2);
		if(!code2) 
			return NULL;
		pd+=4;
		//printf("%04X %04X %04X %04X\n",code,code2,len,len2);
		if((code2==code) && (len2==len))
			return pd;
		pd+=len2;
	}
	
	return NULL;
}
unsigned char *read_txt_pda(FILE *fl) {
	char buf[1024], *in_ptr;
	char *ret, *out_ptr, *len;
	int slen;
	
	if(!fl)
		return NULL;
	if(!(ret=malloc(PDA_SIZE_MAX+6)))
		return NULL;
	out_ptr=ret;
	while(fgets(buf, sizeof(buf), fl)) {
		//puts(buf);
		if((slen=strlen(buf))) {
			if(buf[slen-1]==0x0a) {
				slen--;
				if((slen) && (buf[slen-1]==0x0d))
					slen--;
				buf[slen]=0;
			}
			else if(!feof(fl)) {
				fprintf(stderr,"PDA line too long: %s\n",buf);
				free(ret);
				return NULL;
			}
		} /* len */
		if((slen>=10) && (PDA_SIZE_MAX-(out_ptr-ret)>=6)) {
			len=out_ptr;
			out_ptr+=2;
			if(hex2bin(buf, out_ptr, 2)!=2) {
				fprintf(stderr,"PDA record code invalid: %s\n",buf);
				free(ret);
				return NULL;
			} /* code ok? */
			*((u16*)out_ptr)=s_swap(*((u16*)out_ptr));
			out_ptr+=2;
			in_ptr=buf+5;
			/* parse data */
			while(1) {
				SKIP_SPACES(in_ptr);
				if(!in_ptr[0])
					break;
				else if((out_ptr-ret)>=PDA_SIZE_MAX) {
					fprintf(stderr,"PDA too big: %s\n",in_ptr);
					free(ret);
					return NULL;
				}
				else if(hex2bin(in_ptr, out_ptr, 2)!=2) {
					fprintf(stderr,"PDA data invalid: %s\n",in_ptr);
					free(ret);
					return NULL;
				}
				out_ptr+=2;
				in_ptr+=4;	
			} /* parse */
			*((u16*)len)=host_to_le16((out_ptr-len-2)>>1);
		} /* len ok */
	} /* while */
	((u16*)out_ptr)[0]=host_to_le16(2);
	((u16*)out_ptr)[1]=0;
	((u16*)out_ptr)[2]=0;
	return ret;
}
