/*
 * memory dumping/analyzing tool for hermes wavelan cards
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "../hfw/hermes_mem.h"

#define SKIP_SPACES(a) for(;((a)[0]==0x20) || ((a)[0]==9);(a)++);
#define SKIP_NONSPACES(a) for(;((a)[0]!=0x20) && ((a)[0]!=9) && ((a)[0]);(a)++);

#define SKIP_NUMBERS(a) for(;(((a)[0]>=0x30) && ((a)[0]<=0x39)) || ((a)[0]=='-');(a)++);
#define SKIP_NONNUMBERS(a) for(;((a)[0]) && ((a)[0]!='-') && (((a)[0]<0x30) || ((a)[0]>0x39));(a)++);

#define NEXT_PATTERN(a) SKIP_NONSPACES(a) SKIP_SPACES(a)
#define NEXT_NUMBER(a) SKIP_NUMBERS(a) SKIP_NONNUMBERS(a)

#define PATTERN_OK(a) ((a)[0])
#define NUMBER_OK(a) ((((a)[0]>=0x30) && ((a)[0]<=0x39)) || ((a)[0]=='-'))

#define CMD(a,b) (!(strncasecmp((a),(b),strlen(b))))

int offset(char *str) {
	int count,ret;
	
	for(count=0;(str[count])&&(str[count]!=':');count++);
	if(sscanf(str,"%x",&ret)!=1)
		return -1;
	if(str[count]==':') {
		int r2;
		
		ret*=128;
		if(sscanf(str+count+1,"%x",&r2)!=1)
			return -1;
		ret+=r2;
	}
	return ret;
}

void dump(unsigned char *mem, int ofs, int len) {
	int l,m;
	
	while(len>0) {
		printf("%8X:  ",ofs);
		if(len>16)
			l=16;
		else
			l=len;
		m=l;
		len-=l;
		for(;l;l--) {
			printf("%02X ",mem[ofs++]);
		}
		for(l=16-m;l;l--)
			printf("   ");
		ofs-=m;
		printf(" ");
		for(;m;m--) {
			if((mem[ofs]>=0x20) && (mem[ofs]<=0x7E))
				printf("%c",mem[ofs]);
			else
				printf(".");
			ofs++;
		}
		printf("\n");
	}
}

void diff(unsigned char *mem, int ofs, int len) {
	int count=0,c2;
	
	while(count<len) {
		for(;(count<len) && (mem[ofs+count] == mem[ofs+(4*1024*1024)+count]);count++);
		if(count==len)
			break;
		for(c2=count; (c2<len) && (mem[ofs+c2] != mem[ofs+(4*1024*1024)+c2]);c2++);
		dump(mem+(4*1024*1024),ofs+count,c2-count);
		dump(mem,ofs+count,c2-count);
		//printf("\n");
		count=c2;
	}
}

int main(int argc, char **argv) {
	int fd, ofs, dofs=-1, dlen=-1;
	char buf[200], *bp;
	unsigned char *mem;//[8*1024*1024];

	if(argc<2) {
		printf("usage: %s /dev/orinoco/ethXmem\n",argv[0]);
		return 0;
	}
	
	if(!(mem=malloc(8*1024*1024))) {
		fprintf(stderr,"can't malloc %d\n",8*1024*1024);
		return -1;
	}
	
	if((fd=open(argv[1],O_RDWR))<0) {
		fprintf(stderr,"can't open %s rw\n",argv[1]);
		return -1;
	}
	
	printf("> ");
	fflush(stdout);
	while((!feof(stdin)) && (fgets(buf,sizeof(buf),stdin))) {
		int len=strlen(buf);
		bp=buf;
	
		if(buf[len-1]==0x0a) {
			if(buf[len-2]==0x0d)
				len-=2;
			else
				len--;
			buf[len]=0;
		}
	
		SKIP_SPACES(bp);
		
		if(CMD(bp,"dump")) {
			int r;
			
			/* dump mem */
			NEXT_PATTERN(bp);
			if((ofs=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
			NEXT_PATTERN(bp);
			if(!NUMBER_OK(bp)) {
				printf("invalid length: %s\n",bp);
				goto prompt;
			}
			sscanf(bp,"%d",&len);
			if(len+ofs>(4*1024*1024))
				len=(4*1024*1024)-ofs;
			if((r=lseek(fd,ofs,SEEK_SET))!=ofs) {
				printf("seek failed: %d\n",r);
				goto prompt;
			}
			if((r=read(fd,mem+ofs,len))<len) {
				if(r<=0) {
					printf("read failed: %d\n",r);
					goto prompt;
				}
				else
					len=r;
			}
			dump(mem,ofs, len);
		}
		else if(CMD(bp,"entry")) {
			
			NEXT_PATTERN(bp);
			if((ofs=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
			printf("HERMES_PREPARE_RAMDL: %d\n",ioctl(fd, HERMES_PREPARE_RAMDL,ofs));
		}
		else if(CMD(bp,"reboot")) {
			printf("HERMES_FINISH_RAMDL: %d\n",ioctl(fd, HERMES_FINISH_RAMDL, 0));
		}
		else if(CMD(bp,"boot")) {
			
			NEXT_PATTERN(bp);
			if((ofs=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
			printf("HERMES_PREPARE_RAMDL: %d\n",ioctl(fd, HERMES_PREPARE_RAMDL,ofs));
			printf("HERMES_FINISH_RAMDL: %d\n",ioctl(fd, HERMES_FINISH_RAMDL, 0));
		}
		else if(CMD(bp,"copy")) {
			int ofs2;
			unsigned char *membuf, *memptr;
			int res;
			
			NEXT_PATTERN(bp);
			if((ofs=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
			
			NEXT_PATTERN(bp);
			if((ofs2=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
			
			NEXT_PATTERN(bp);
			if(!NUMBER_OK(bp)) {
				printf("invalid length: %s\n",bp);
				goto prompt;
			}
			sscanf(bp,"%d",&len);
			
			if(len+ofs>(4*1024*1024))
				len=(4*1024*1024)-ofs;
			if(len+ofs2>(4*1024*1024))
				len=(4*1024*1024)-ofs2;
			
			if(!(membuf=malloc(len))) {
				printf("malloc(len=%d) failed!\n",len);
				goto prompt;
			}
			memptr=membuf;
			if((res=lseek(fd,ofs,SEEK_SET))!=ofs) {
					printf("seek failed: %d\n",res);
					goto copy_free;
			}
			while(memptr<membuf+len) {
				if((res=read(fd,memptr,(((membuf+len)-memptr)>=len) ? (len) : ((membuf+len)-memptr)))<=0) {
					printf("read failed: %d\n",res);
					goto copy_free;
				}
				memptr+=res;
			}
			if((res=lseek(fd,ofs2,SEEK_SET))!=ofs2) {
					printf("seek failed: %d\n",res);
					goto copy_free;
			}
			memptr=membuf;
			while(memptr<membuf+len) {
				if((res=write(fd,memptr,(((membuf+len)-memptr)>=len) ? (len) : ((membuf+len)-memptr)))<=0) {
					printf("write failed: %d\n",res);
					goto copy_free;
				}
				memptr+=res;
			}
			copy_free:
			free(membuf);
		}
		else if(CMD(bp,"diff")) {
			int r;
			
			/* diff mem */
			NEXT_PATTERN(bp);
			
			if(PATTERN_OK(bp)) {
				if((dofs=offset(bp))<0) {
					printf("invalid offset: %s\n",bp);
					goto prompt;
				}
				NEXT_PATTERN(bp);
				if(!NUMBER_OK(bp)) {
					printf("invalid length: %s\n",bp);
					goto prompt;
				}
				sscanf(bp,"%d",&dlen);
				if(dlen+dofs>(4*1024*1024))
					dlen=(4*1024*1024)-dofs;
				if((r=lseek(fd,dofs,SEEK_SET))!=dofs) {
					printf("seek failed: %d\n",r);
					goto prompt;
				}
				if((r=read(fd,mem+(4*1024*1024)+dofs,dlen))<dlen) {
					if(r<=0) {
						printf("read failed: %d\n",r);
						goto prompt;
					}
					else
						dlen=r;
				}
			}
			
			else if((dofs>=0) && (dlen>0)) {
				if((r=lseek(fd,dofs,SEEK_SET))!=dofs) {
					printf("seek failed: %d\n",r);
					goto prompt;
				}
				if((r=read(fd,mem+dofs,dlen))<dlen) {
					if(r<=0) {
						printf("read failed: %d\n",r);
						goto prompt;
					}
					else
						dlen=r;
				}
				/* compare */
				diff(mem,dofs,dlen);
				memcpy(mem+(4*1024*1024)+dofs,mem+dofs,dlen);
			}
			else
				printf("no range loaded yet\n");
		}
		else if(CMD(bp,"write")) {
			/* write mem */
			NEXT_PATTERN(bp);
			if((ofs=offset(bp))<0) {
				printf("invalid offset: %s\n",bp);
				goto prompt;
			}
		}
		else if((CMD(bp,"quit")) || (CMD(bp,"exit")))
			break;
		else {
			NEXT_PATTERN(bp);
			if(*bp)
				printf("cmd unknown: %s\n",bp);
		}
	
	prompt:
		printf("> ");
		fflush(stdout);
	}
	close(fd);
	
	return 0;
}
