/*
 * userspace memory/rid dumping tool for hermes wavelan cards
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */

#include <unistd.h>
#include <sys/io.h>
#include <stdio.h>
#include <stdlib.h>

#define LEN 0x40

#define		HERMES_CMD			(0x00)
#define		HERMES_PARAM0			(0x02)
#define		HERMES_PARAM1			(0x04)
#define		HERMES_PARAM2			(0x06)
#define		HERMES_STATUS			(0x08)
#define		HERMES_RESP0			(0x0A)
#define		HERMES_RESP1			(0x0C)
#define		HERMES_RESP2			(0x0E)
#define		HERMES_INFOFID			(0x10)
#define		HERMES_RXFID			(0x20)
#define		HERMES_ALLOCFID			(0x22)
#define		HERMES_TXCOMPLFID		(0x24)
#define		HERMES_SELECT0			(0x18)
#define		HERMES_OFFSET0			(0x1C)
#define		HERMES_DATA0			(0x36)
#define		HERMES_SELECT1			(0x1A)
#define		HERMES_OFFSET1			(0x1E)
#define		HERMES_DATA1			(0x38)
#define		HERMES_EVSTAT			(0x30)
#define		HERMES_INTEN			(0x32)
#define		HERMES_EVACK			(0x34)
#define		HERMES_CONTROL			(0x14)
#define		HERMES_SWSUPPORT0		(0x28)
#define		HERMES_SWSUPPORT1		(0x2A)
#define		HERMES_SWSUPPORT2		(0x2C)
#define		HERMES_AUXPAGE			(0x3A)
#define		HERMES_AUXOFFSET		(0x3C)
#define		HERMES_AUXDATA			(0x3E)

#define		HERMES_AUX_MAGIC0		0xfe01
#define		HERMES_AUX_MAGIC1		0xdc23
#define		HERMES_AUX_MAGIC2		0xba45

#define hermes_read_reg(base, off) (inw((base) + (off)))
#define hermes_write_reg(base, off, val) (outw((val), (base) + (off)))

#define hermes_read_regn(base, name) (hermes_read_reg((base), HERMES_##name))
#define hermes_write_regn(base, name, val) (hermes_write_reg((base), HERMES_##name, (val)))

unsigned short read_fid(int base, unsigned short fid, unsigned short *buf, unsigned short len) {
	unsigned short ret;
	
	hermes_write_regn(base, PARAM0, fid);
	hermes_write_regn(base, CMD, 0x21);
	usleep(1000000);
	ret=hermes_read_regn(base, EVSTAT);
	printf("EVSTAT: %04X\n",ret);
	if(ret!=0x8010)
		return 0;
	ret=hermes_read_regn(base, STATUS);
	printf("STATUS: %04X\n",ret);
	if(ret!=0x21)
		return 0;
	hermes_write_regn(base, EVACK, 0x10);
	hermes_write_regn(base, SELECT1, fid);
	hermes_write_regn(base, OFFSET1, 0);
	ret=0;
	while(ret<len) {
		*buf=hermes_read_regn(base, DATA1);
		buf++;
		ret++;
	}
	return ret;
}

void dump(unsigned short *buf, int len) {
	int count;
	
	for(count=0;count<len;count++)
		printf("%04X ",buf[count]);
	printf("\n");
}

int main(int argc, char **argv) {
	int base=0x100;
	unsigned short ret;
	unsigned long left=8*1024*1024;
	unsigned short buf[512];
	
	if(ioperm(base,LEN,1)) {
		fprintf(stderr,"can't access 0x%03X-0x%03X\n",base,base+LEN);
		return -1;
	}

 	if(argc==3) {
		ret=atoi(argv[2]);
		sscanf(argv[1],"%lx",&left);
		read_fid(base,left&0xFFFF,buf,4+ret);
		dump(buf+4,ret);
	}
	else {
		hermes_write_regn(base, PARAM0, HERMES_AUX_MAGIC0);
		hermes_write_regn(base, PARAM1, HERMES_AUX_MAGIC1);
		hermes_write_regn(base, PARAM2, HERMES_AUX_MAGIC2);
		hermes_write_regn(base, CONTROL, 0x8000);
		usleep(100000);
		ret=hermes_read_regn(base, CONTROL);
		fprintf(stderr,"CONTROL: %04X\n",ret);
		if(ret!=0xc000)
			goto out;
		hermes_write_regn(base, AUXPAGE, 0);
		hermes_write_regn(base, AUXOFFSET, 0);
	
		for(;left;left-=2) {
			ret=hermes_read_regn(base, AUXDATA);
			write(1,&ret,2);
		}
	
		hermes_write_regn(base, PARAM0, HERMES_AUX_MAGIC0);
		hermes_write_regn(base, PARAM1, HERMES_AUX_MAGIC1);
		hermes_write_regn(base, PARAM2, HERMES_AUX_MAGIC2);
		hermes_write_regn(base, CONTROL, 0x4000);
		usleep(100000);
		ret=hermes_read_regn(base, CONTROL);
		fprintf(stderr,"CONTROL: %04X\n",ret);
	}
		
out:
	ioperm(base,LEN,0);
	
	return 0;
}
